@@ -5,8 +5,13 @@ import traceback
 
 import ispyb.interface.connection
 import mysql.connector
-from ispyb import ConnectionError, NoResult, ReadWriteError
-from mysql.connector.errors import DatabaseError, DataError, Error, InterfaceError
+from ispyb import ISPyBException, ConnectionError, NoResult, ReadWriteError
+from mysql.connector.errors import (
+    DatabaseError,
+    DataError,
+    IntegrityError,
+    InterfaceError,
+)
 
 
 class ISPyBMySQLSPConnector(ispyb.interface.connection.IF):
@@ -95,6 +100,10 @@ class ISPyBMySQLSPConnector(ispyb.interface.connection.IF):
                 raise ReadWriteError(
                     "DataError({0}): {1}".format(e.errno, traceback.format_exc())
                 )
+            except IntegrityError as e:
+                raise ReadWriteError(
+                    "IntegrityError({0}): {1}".format(e.errno, traceback.format_exc())
+                )
             finally:
                 cursor.close()
         if result_args is not None and len(result_args) > 0:
@@ -156,6 +165,10 @@ class ISPyBMySQLSPConnector(ispyb.interface.connection.IF):
                 raise ReadWriteError(
                     "DataError({0}): {1}".format(e.errno, traceback.format_exc())
                 )
+            except IntegrityError as e:
+                raise ReadWriteError(
+                    "IntegrityError({0}): {1}".format(e.errno, traceback.format_exc())
+                )
             result = None
             rs = cursor.fetchone()
             if len(rs) > 0:
@@ -165,3 +178,12 @@ class ISPyBMySQLSPConnector(ispyb.interface.connection.IF):
                     result = rs[0]
             cursor.close()
         return result
+
+    def set_role(self, role):
+        cursor = self.create_cursor()
+        try:
+            cursor.execute("SET ROLE %s" % role)
+        except DatabaseError as e:
+            raise ISPyBException(
+                "DatabaseError({0}): {1}".format(e.errno, traceback.format_exc())
+            )
